@@ -1,8 +1,8 @@
 package CPAN::Reporter;
 use strict;
 use vars qw/$VERSION/;
-$VERSION = '1.16_51'; 
-$VERSION = eval $VERSION;
+$VERSION = '1.18_05';
+$VERSION = eval $VERSION; ## no critic
 
 use Config;
 use CPAN ();
@@ -22,7 +22,7 @@ use CPAN::Reporter::Config ();
 use CPAN::Reporter::History ();
 use CPAN::Reporter::PrereqCheck ();
 
-use constant MAX_OUTPUT_LENGTH => 50_000;
+use constant MAX_OUTPUT_LENGTH => 1_000_000;
 
 #--------------------------------------------------------------------------#
 # create temp lib dir for Devel::Autoflush
@@ -31,12 +31,12 @@ use constant MAX_OUTPUT_LENGTH => 50_000;
 
 require Devel::Autoflush;
 # directory fixture
-my $Autoflush_Lib = tempdir( 
-  "CPAN-Reporter-lib-XXXX", TMPDIR => 1, CLEANUP => 1 
+my $Autoflush_Lib = tempdir(
+  "CPAN-Reporter-lib-XXXX", TMPDIR => 1, CLEANUP => 1
 );
 # copy Devel::Autoflush to directory or clear autoflush_lib variable
-_file_copy_quiet( 
-  $INC{'Devel/Autoflush.pm'}, 
+_file_copy_quiet(
+  $INC{'Devel/Autoflush.pm'},
   File::Spec->catfile( $Autoflush_Lib, qw/Devel Autoflush.pm/ )
 ) or undef $Autoflush_Lib;
 
@@ -45,20 +45,20 @@ _file_copy_quiet(
 #--------------------------------------------------------------------------#
 
 sub configure {
-    goto &CPAN::Reporter::Config::_configure; 
+    goto &CPAN::Reporter::Config::_configure;
 }
 
-sub grade_make { 
+sub grade_make {
     my @args = @_;
-    my $result = _init_result( 'make', @args ) or return; 
+    my $result = _init_result( 'make', @args ) or return;
     _compute_make_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->myprint( 
+        $CPAN::Frontend->myprint(
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
             "Test report will not be sent"
         );
-        CPAN::Reporter::History::_record_history( $result ) 
+        CPAN::Reporter::History::_record_history( $result )
             if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
@@ -70,15 +70,15 @@ sub grade_make {
 
 sub grade_PL {
     my @args = @_;
-    my $result = _init_result( 'PL', @args ) or return; 
+    my $result = _init_result( 'PL', @args ) or return;
     _compute_PL_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->myprint( 
+        $CPAN::Frontend->myprint(
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
             "Test report will not be sent"
         );
-        CPAN::Reporter::History::_record_history( $result ) 
+        CPAN::Reporter::History::_record_history( $result )
             if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
@@ -90,15 +90,15 @@ sub grade_PL {
 
 sub grade_test {
     my @args = @_;
-    my $result = _init_result( 'test', @args ) or return; 
+    my $result = _init_result( 'test', @args ) or return;
     _compute_test_grade($result);
     if ( $result->{grade} eq 'discard' ) {
-        $CPAN::Frontend->myprint( 
+        $CPAN::Frontend->myprint(
             "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
             "Test report will not be sent"
         );
-        CPAN::Reporter::History::_record_history( $result ) 
+        CPAN::Reporter::History::_record_history( $result )
             if not CPAN::Reporter::History::_is_duplicate( $result );
     }
     else {
@@ -111,12 +111,12 @@ sub grade_test {
 sub record_command {
     my ($command, $timeout) = @_;
 
-    # XXX refactor this! 
+    # XXX refactor this!
     # Get configuration options
     if ( -r CPAN::Reporter::Config::_get_config_file() ) {
         my $config_obj = CPAN::Reporter::Config::_open_config_file();
         my $config;
-        $config = CPAN::Reporter::Config::_get_config_options( $config_obj ) 
+        $config = CPAN::Reporter::Config::_get_config_options( $config_obj )
             if $config_obj;
 
         $timeout ||= $config->{command_timeout}; # might still be undef
@@ -126,7 +126,7 @@ sub record_command {
 
     my $temp_out = _temp_filename( 'CPAN-Reporter-TO-' );
 
-    # Teeing a command loses its exit value so we must wrap the command 
+    # Teeing a command loses its exit value so we must wrap the command
     # and print the exit code so we can read it off of output
     my $wrap_code;
     if ( $timeout ) {
@@ -151,21 +151,21 @@ HERE
 
     $wrapper_fh->print( $wrap_code );
     $wrapper_fh->close;
-    
+
     # tee the command wrapper
     my $tee_input = Probe::Perl->find_perl_interpreter() .  " $wrapper_name";
     $tee_input .= " $redirect" if defined $redirect;
     {
       # ensure autoflush if we can
-      local $ENV{PERL5OPT} = _get_perl5opt();
+      local $ENV{PERL5OPT} = _get_perl5opt() if _is_PL($command);
       tee($tee_input, { stderr => 1 }, $temp_out);
     }
 
     # read back the output
     my $output_fh = IO::File->new($temp_out, "r");
     if ( !$output_fh ) {
-        $CPAN::Frontend->mywarn( 
-            "CPAN::Reporter: couldn't read command results for '$cmd'\n" 
+        $CPAN::Frontend->mywarn(
+            "CPAN::Reporter: couldn't read command results for '$cmd'\n"
         );
         return;
     }
@@ -173,10 +173,10 @@ HERE
     $output_fh->close;
 
     # cleanup
-    unlink $wrapper_name, $temp_out;
+    unlink $wrapper_name, $temp_out unless $ENV{PERL_CR_NO_CLEANUP};
 
     if ( ! @cmd_output ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->mywarn(
             "CPAN::Reporter: didn't capture command results for '$cmd'\n"
         );
         return;
@@ -188,16 +188,16 @@ HERE
         ($exit_value) = $cmd_output[-1] =~ m{exited with ([-0-9]+)};
         pop @cmd_output;
     }
-    
+
     # bail out on some errors
     if ( ! defined $exit_value ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->mywarn(
             "CPAN::Reporter: couldn't determine exit value for '$cmd'\n"
         );
         return;
     }
     elsif ( $exit_value == -1 ) {
-        $CPAN::Frontend->mywarn( 
+        $CPAN::Frontend->mywarn(
             "CPAN::Reporter: couldn't execute '$cmd'\n"
         );
         return;
@@ -224,7 +224,7 @@ sub _compute_make_grade {
     my $result = shift;
     my ($grade,$msg);
     if ( $result->{exit_value} ) {
-        $result->{grade} = "fail";
+        $result->{grade} = "unknown";
         $result->{grade_msg} = "Stopped with an error"
     }
     else {
@@ -233,9 +233,8 @@ sub _compute_make_grade {
     }
 
     _downgrade_known_causes( $result );
-    
-    $result->{success} =  $result->{grade} eq 'pass'
-                       || $result->{grade} eq 'unknown';
+
+    $result->{success} =  $result->{grade} eq 'pass';
     return;
 }
 
@@ -247,7 +246,7 @@ sub _compute_PL_grade {
     my $result = shift;
     my ($grade,$msg);
     if ( $result->{exit_value} ) {
-        $result->{grade} = "fail";
+        $result->{grade} = "unknown";
         $result->{grade_msg} = "Stopped with an error"
     }
     else {
@@ -256,9 +255,8 @@ sub _compute_PL_grade {
     }
 
     _downgrade_known_causes( $result );
-    
-    $result->{success} =  $result->{grade} eq 'pass'
-                       || $result->{grade} eq 'unknown';
+
+    $result->{success} =  $result->{grade} eq 'pass';
     return;
 }
 
@@ -267,18 +265,18 @@ sub _compute_PL_grade {
 #
 # Don't shortcut to unknown unless _has_tests because a custom
 # Makefile.PL or Build.PL might define tests in a non-standard way
-# 
+#
 # With test.pl and 'make test', any t/*.t might pass Test::Harness, but
 # test.pl might still fail, or there might only be test.pl,
 # so use exit code directly
 #
 # Likewise, if we have recursive Makefile.PL, then we don't trust the
-# reverse-order parsing and should just take the exit code directly 
+# reverse-order parsing and should just take the exit code directly
 #
 # Otherwise, parse in reverse order for Test::Harness output or a couple
-# other significant strings and stop after the first match.  Going in 
-# reverse and stopping is done to (hopefully) avoid picking up spurious 
-# results from any test output.  But then we have to check for 
+# other significant strings and stop after the first match.  Going in
+# reverse and stopping is done to (hopefully) avoid picking up spurious
+# results from any test output.  But then we have to check for
 # unsupported OS strings in case those were printed but were not fatal.
 #--------------------------------------------------------------------------#
 
@@ -287,7 +285,7 @@ sub _compute_test_grade {
     my ($grade,$msg);
     my $output = $result->{output};
 
-    # In some cases, get a result straight from the exit code 
+    # In some cases, get a result straight from the exit code
     if ( $result->{is_make} && ( -f "test.pl" || _has_recursive_make() ) ) {
         if ( $result->{exit_value} ) {
             $grade = "fail";
@@ -301,7 +299,7 @@ sub _compute_test_grade {
     # Otherwise, get a result from Test::Harness output
     else {
         # figure out the right harness parser
-        _expand_result( $result ); 
+        _expand_result( $result );
         my $harness_version = $result->{toolchain}{'Test::Harness'}{have};
         my $harness_parser = CPAN::Version->vgt($harness_version, '2.99_01')
                     ? \&_parse_tap_harness
@@ -342,7 +340,7 @@ sub _compute_test_grade {
 #--------------------------------------------------------------------------#
 # _dispatch_report
 #
-# Set up Test::Reporter and prompt user for CC, edit, send
+# Set up Test::Reporter and prompt user for edit, send
 #--------------------------------------------------------------------------#
 
 sub _dispatch_report {
@@ -356,33 +354,38 @@ sub _dispatch_report {
     # Get configuration options
     my $config_obj = CPAN::Reporter::Config::_open_config_file();
     my $config;
-    $config = CPAN::Reporter::Config::_get_config_options( $config_obj ) 
+    $config = CPAN::Reporter::Config::_get_config_options( $config_obj )
         if $config_obj;
     if ( ! $config->{email_from} ) {
         $CPAN::Frontend->mywarn( << "EMAIL_REQUIRED");
-        
+
 CPAN::Reporter: required 'email_from' option missing an email address, so
 test report will not be sent. See documentation for configuration details.
 
+Even for non-email transports (e.g. Metabase, File or Socket) this email
+address will show up in the report and help identify the tester.
+This is required for compatibility with tools that process legacy reports
+for analysis.
+
 EMAIL_REQUIRED
         return;
     }
-        
+
     # Need to know if this is a duplicate
     my $is_duplicate = CPAN::Reporter::History::_is_duplicate( $result );
 
-    # Abort if the distribution name is not formatted according to 
+    # Abort if the distribution name is not formatted according to
     # CPAN Testers requirements: Dist-Name-version.suffix
     # Regex from CPAN-Testers should extract name, separator, version
     # and extension
-    my @format_checks = $result->{dist_basename} =~ 
+    my @format_checks = $result->{dist_basename} =~
         m{(.+)([\-\_])(v?\d.*)(\.(?:tar\.(?:gz|bz2)|tgz|zip))$}i;
     ;
     if ( ! grep { length } @format_checks ) {
         $CPAN::Frontend->mywarn( << "END_BAD_DISTNAME");
-        
-CPAN::Reporter: the distribution name '$result->{dist_basename}' does not 
-appear to be packaged according to CPAN tester guidelines. Perhaps it is 
+
+CPAN::Reporter: the distribution name '$result->{dist_basename}' does not
+appear to be packaged according to CPAN tester guidelines. Perhaps it is
 not a normal CPAN distribution.
 
 Test report will not be sent.
@@ -391,7 +394,7 @@ END_BAD_DISTNAME
 
         # record this as a discard, instead
         $result->{grade} = 'discard';
-        CPAN::Reporter::History::_record_history( $result ) 
+        CPAN::Reporter::History::_record_history( $result )
             if not $is_duplicate;
         return;
     }
@@ -411,7 +414,7 @@ END_BAD_DISTNAME
             next if $dist_id !~ /$pattern/i;
             # if it matches, warn and return
             $CPAN::Frontend->myprint( << "END_SKIP_DIST" );
-CPAN::Reporter: '$dist_id' matched against the send_skipfile.  
+CPAN::Reporter: '$dist_id' matched against the send_skipfile.
 
 Test report will not be sent.
 
@@ -425,6 +428,10 @@ END_SKIP_DIST
     my $tr = Test::Reporter->new;
     $tr->grade( $result->{grade} );
     $tr->distribution( $result->{dist_name}  );
+    # Older Test::Reporter doesn't support distfile, but we need it for
+    # Metabase transport
+    $tr->distfile( $result->{dist}->pretty_id )
+      if $Test::Reporter::VERSION >= 1.54;
 
     # Skip if duplicate and not sending duplicates
     if ( $is_duplicate ) {
@@ -437,7 +444,7 @@ CPAN::Reporter: this appears to be a duplicate report for the $phase phase:
 Test report will not be sent.
 
 DUPLICATE_REPORT
-            
+
             return;
         }
     }
@@ -463,11 +470,10 @@ DUPLICATE_REPORT
         my @mx = split " ", $config->{smtp_server};
         $tr->mx( \@mx );
     }
-    
+
     # Populate the test report
     $tr->comments( _report_text( $result ) );
     $tr->via( 'CPAN::Reporter ' . $CPAN::Reporter::VERSION );
-    my @cc = _should_copy_author( $result, $config );
 
     # prompt for editing report
     if ( _prompt( $config, "edit_report", $tr->grade ) =~ /^y/ ) {
@@ -475,16 +481,16 @@ DUPLICATE_REPORT
         local $ENV{VISUAL} = $editor if $editor; ## no critic
         $tr->edit_comments;
     }
-    
+
     # send_*_report can override send_report
     my $send_config = defined $config->{"send_$phase\_report"}
                     ? "send_$phase\_report"
                     : "send_report" ;
     if ( _prompt( $config, $send_config, $tr->grade ) =~ /^y/ ) {
-        $CPAN::Frontend->myprint( "CPAN::Reporter: sending test report with '" . $tr->grade . 
-              "' to " . join(q{, }, $tr->address, @cc) . "\n");
-        if ( $tr->send( @cc ) ) {
-            CPAN::Reporter::History::_record_history( $result ) 
+        $CPAN::Frontend->myprint( "CPAN::Reporter: sending test report with '" . $tr->grade .
+              "' via " . $transport_args[0] . "\n");
+        if ( $tr->send() ) {
+            CPAN::Reporter::History::_record_history( $result )
                 if not $is_duplicate;
         }
         else {
@@ -518,7 +524,7 @@ sub _downgrade_known_causes {
     return if $grade eq 'pass' && $result->{phase} ne 'PL';
 
     # get prereqs
-    _expand_result( $result ); 
+    _expand_result( $result );
 
     # if process was halted with a signal, just set for discard and return
     if ( $result->{exit_value} & 127 ) {
@@ -553,24 +559,44 @@ sub _downgrade_known_causes {
         $grade = 'na';
         $msg = 'This platform is not supported';
     }
+    # check for Makefile without 'test' target; there are lots
+    # of variations on the error message, e.g. "target test", "target 'test'",
+    # "'test'", "`test'" and so on.
+    elsif (
+      $result->{is_make} && $result->{phase} eq 'test' && ! _has_test_target()
+    ) {
+        $grade = 'unknown';
+        $msg = 'No make test target';
+    }
     # check the prereq report for missing or failure flag '!'
     elsif ( $grade ne 'pass' && $result->{prereq_pm} =~ m{n/a}ims ) {
         $grade = 'discard';
-        $msg = 'Prerequisite missing';
+        $msg = "Prerequisite missing:\n$result->{prereq_pm}";
     }
     elsif ( $grade ne 'pass' && $result->{prereq_pm} =~ m{^\s+!}ims ) {
         $grade = 'discard';
-        $msg = 'Prerequisite version too low';
+        $msg = "Prerequisite version too low:\n$result->{prereq_pm}";
     }
-    # in PL stage -- if pass but no Makefile or Build, then this should 
+    # in PL stage -- if pass but no Makefile or Build, then this should
     # be recorded as a discard
-    elsif ( $result->{phase} eq 'PL' && $grade eq 'pass' 
-         && ! -f 'Makefile' && ! -f 'Build' 
+    elsif ( $result->{phase} eq 'PL' && $grade eq 'pass'
+         && ! -f 'Makefile' && ! -f 'Build'
     ) {
         $grade = 'discard';
         $msg = 'No Makefile or Build file found';
     }
-    
+    elsif ( $result->{command} =~ /Build.*?-j/ ) {
+        $grade = 'discard';
+        $msg = '-j is not a valid option for Module::Build (upgrade your CPAN.pm)';
+    }
+    elsif (
+      $result->{is_make} && $result->{phase} eq 'make' &&
+      grep { /Makefile out-of-date with respect to Makefile.PL/ } @$output
+    ) {
+        $grade = 'discard';
+        $msg = 'Makefile out-of-date';
+    }
+
     # store results
     $result->{grade} = $grade;
     $result->{grade_msg} = $msg;
@@ -588,7 +614,11 @@ sub _expand_result {
     my $result = shift;
     return if $result->{expanded}++; # only do this once
     $result->{prereq_pm} = _prereq_report( $result->{dist} );
-    $result->{env_vars} = _env_report();
+    {
+      # mirror PERL5OPT as in record_command
+      local $ENV{PERL5OPT} = _get_perl5opt() if _is_PL($result->{command});
+      $result->{env_vars} = _env_report();
+    }
     $result->{special_vars} = _special_vars_report();
     $result->{toolchain_versions} = _toolchain_report( $result );
     $result->{perl_version} = CPAN::Reporter::History::_format_perl_version();
@@ -622,8 +652,6 @@ my @env_vars= qw(
 
 sub _env_report {
     my @vars_found;
-    # mirror PERL5OPT as in record_command
-    local $ENV{PERL5OPT} = _get_perl5opt();
     for my $var ( @env_vars ) {
         if ( $var =~ m{^/(.+)/$} ) {
             push @vars_found, grep { /$1/ } keys %ENV;
@@ -654,11 +682,11 @@ sub _file_copy_quiet {
   mkpath( dirname($target) ) or return;
   # read source
   local *FH;
-  open FH, "<$source" or return;
+  open FH, "<$source" or return; ## no critic
   my $pm_guts = do { local $/; <FH> };
   close FH;
   # write target
-  open FH, ">$target" or return;
+  open FH, ">$target" or return; ## no critic
   print FH $pm_guts;
   close FH;
   return 1;
@@ -672,14 +700,16 @@ sub _get_perl5opt {
   my $perl5opt = $ENV{PERL5OPT} || q{};
   if ( $Autoflush_Lib ) {
     $perl5opt .= q{ } if length $perl5opt;
-    $perl5opt .= "-I$Autoflush_Lib -MDevel::Autoflush";
+    $perl5opt .= "-I$Autoflush_Lib " if $] >= 5.008;
+    $perl5opt .= "-MDevel::Autoflush";
   }
+  return $perl5opt;
 }
 
 #--------------------------------------------------------------------------#
 # _has_recursive_make
 #
-# Ignore Makefile.PL in t directories 
+# Ignore Makefile.PL in t directories
 #--------------------------------------------------------------------------#
 
 sub _has_recursive_make {
@@ -699,7 +729,16 @@ sub _has_recursive_make {
 }
 
 #--------------------------------------------------------------------------#
-# _init_result -- create and return a hash of values for use in 
+# _has_test_target
+#--------------------------------------------------------------------------#
+
+sub _has_test_target {
+  my $fh = IO::File->new("Makefile") or return;
+  return scalar grep { /^test[ ]*:/ } <$fh>;
+}
+
+#--------------------------------------------------------------------------#
+# _init_result -- create and return a hash of values for use in
 # report evaluation and dispatch
 #
 # takes same argument format as grade_*()
@@ -707,7 +746,7 @@ sub _has_recursive_make {
 
 sub _init_result {
     my ($phase, $dist, $system_command, $output, $exit_value) = @_;
-    
+
     unless ( defined $output && defined $exit_value ) {
         my $missing;
         if ( ! defined $output && ! defined $exit_value ) {
@@ -758,6 +797,15 @@ sub _is_make {
     return $command =~ m{\b(?:\S*make|Makefile.PL)\b}ims ? 1 : 0;
 }
 
+#--------------------------------------------------------------------------#
+# _is_PL
+#--------------------------------------------------------------------------#
+
+sub _is_PL {
+  my $command = shift;
+  return $command =~ m{\b(?:Makefile|Build)\.PL\b}ims ? 1 : 0;
+}
+
 #--------------------------------------------------------------------------#
 # _max_length
 #--------------------------------------------------------------------------#
@@ -771,10 +819,10 @@ sub _max_length {
     return $max;
 }
 
-    
+
 #--------------------------------------------------------------------------#
 # _parse_tap_harness
-# 
+#
 # As of Test::Harness 2.99_02, the final line is provided by TAP::Harness
 # as "Result: STATUS" where STATUS is "PASS", "FAIL" or "NOTESTS"
 #--------------------------------------------------------------------------#
@@ -803,7 +851,7 @@ sub _parse_tap_harness {
 # _parse_test_harness
 #
 # Output strings taken from Test::Harness::
-# _show_results()  -- for versions < 2.57_03 
+# _show_results()  -- for versions < 2.57_03
 # get_results()    -- for versions >= 2.57_03
 #--------------------------------------------------------------------------#
 
@@ -838,15 +886,15 @@ my @prereq_sections = qw/requires build_requires configure_requires/;
 sub _prereq_report {
     my $dist = shift;
     my (%need, %have, %prereq_met, $report);
-    
+
     # Extract requires/build_requires from CPAN dist
     my $prereq_pm = $dist->prereq_pm;
     if ( ref $prereq_pm eq 'HASH' ) {
         # is it the new CPAN style with requires/build_requires?
         if (join(q{ }, sort keys %$prereq_pm) eq "build_requires requires") {
-            $need{requires} = $prereq_pm->{requires} 
+            $need{requires} = $prereq_pm->{requires}
                 if  ref $prereq_pm->{requires} eq 'HASH';
-            $need{build_requires} = $prereq_pm->{build_requires} 
+            $need{build_requires} = $prereq_pm->{build_requires}
                 if ref $prereq_pm->{build_requires} eq 'HASH';
         }
         else {
@@ -860,12 +908,12 @@ sub _prereq_report {
       if ( -f $meta_yml ) {
         my @yaml = eval { Parse::CPAN::Meta::LoadFile($meta_yml) };
         if ( $@ ) {
-          $CPAN::Frontend->mywarn( 
+          $CPAN::Frontend->mywarn(
             "CPAN::Reporter: error parsing META.yml\n"
           );
         }
-        if (  ref $yaml[0] eq 'HASH' && 
-              ref $yaml[0]{configure_requires} eq 'HASH' 
+        if (  ref $yaml[0] eq 'HASH' &&
+              ref $yaml[0]{configure_requires} eq 'HASH'
         ) {
           $need{configure_requires} = $yaml[0]{configure_requires};
         }
@@ -883,8 +931,8 @@ sub _prereq_report {
             $prereq_met{$section}{$mod} = $prereq_results->{$mod}{met};
         }
     }
-    
-    # find formatting widths 
+
+    # find formatting widths
     my ($name_width, $need_width, $have_width) = (6, 4, 4);
     for my $section ( @prereq_sections ) {
         for my $module ( keys %{ $need{$section} } ) {
@@ -897,28 +945,29 @@ sub _prereq_report {
         }
     }
 
-    my $format_str = 
+    my $format_str =
         "  \%1s \%-${name_width}s \%-${need_width}s \%-${have_width}s\n";
 
     # generate the report
     for my $section ( @prereq_sections ) {
-        if ( keys %{ $need{$section} } ) {
-            $report .= "$section:\n\n";
-            $report .= sprintf( $format_str, " ", qw/Module Need Have/ );
-            $report .= sprintf( $format_str, " ", 
-                                 "-" x $name_width, 
-                                 "-" x $need_width,
-                                 "-" x $have_width );
-        }
-        for my $module ( sort { lc $a cmp lc $b } keys %{ $need{$section} } ) {
-            my $need = $need{$section}{$module};
-            my $have = $have{$section}{$module};
-            my $bad = $prereq_met{$section}{$module} ? " " : "!";
-            $report .= 
-                sprintf( $format_str, $bad, $module, $need, $have);
+      if ( keys %{ $need{$section} } ) {
+        $report .= "$section:\n\n";
+        $report .= sprintf( $format_str, " ", qw/Module Need Have/ );
+        $report .= sprintf( $format_str, " ",
+          "-" x $name_width,
+          "-" x $need_width,
+          "-" x $have_width );
+        for my $module (sort {lc $a cmp lc $b} keys %{ $need{$section} } ) {
+          my $need = $need{$section}{$module};
+          my $have = $have{$section}{$module};
+          my $bad = $prereq_met{$section}{$module} ? " " : "!";
+          $report .=
+          sprintf( $format_str, $bad, $module, $need, $have);
         }
+        $report .= "\n";
+      }
     }
-    
+
     return $report || "    No requirements found\n";
 }
 
@@ -953,13 +1002,13 @@ sub _prompt {
     my $action = $dispatch->{$grade} || $dispatch->{default};
     my $intro = $spec{$option}{prompt} . $extra . " (yes/no)";
     my $prompt;
-    if     ( $action =~ m{^ask/yes}i ) { 
+    if     ( $action =~ m{^ask/yes}i ) {
         $prompt = CPAN::Shell::colorable_makemaker_prompt( $intro, "yes" );
     }
     elsif  ( $action =~ m{^ask(/no)?}i ) {
         $prompt = CPAN::Shell::colorable_makemaker_prompt( $intro, "no" );
     }
-    else { 
+    else {
         $prompt = $action;
     }
     return lc $prompt;
@@ -983,31 +1032,31 @@ If you think this report is invalid, please consult the CPAN Testers Wiki
 for suggestions on how to avoid getting FAIL reports for missing library
 or binary dependencies, unsupported operating systems, and so on:
 
-http://cpantest.grango.org/wiki/CPANAuthorNotes
+http://wiki.cpantesters.org/wiki/CPANAuthorNotes
 HERE
 
     'unknown' => <<'HERE',
 Thank you for uploading your work to CPAN.  However, attempting to
-test your distribution gave an inconclusive result.  
+test your distribution gave an inconclusive result.
 
-This could be because you did not define tests, tests could not be 
-found, because your tests were interrupted before they finished, or 
-because the results of the tests could not be parsed.  You may wish to 
-consult the CPAN Testers Wiki:
+This could be because your distribution had an error during the make/build
+stage, did not define tests, tests could not be found, because your tests were
+interrupted before they finished, or because the results of the tests could not
+be parsed.  You may wish to consult the CPAN Testers Wiki:
 
-http://cpantest.grango.org/wiki/CPANAuthorNotes
+http://wiki.cpantesters.org/wiki/CPANAuthorNotes
 HERE
 
     'na' => <<'HERE',
-Thank you for uploading your work to CPAN.  While attempting to build or test 
-this distribution, the distribution signaled that support is not available 
-either for this operating system or this version of Perl.  Nevertheless, any 
-diagnostic output produced is provided below for reference.  If this is not 
+Thank you for uploading your work to CPAN.  While attempting to build or test
+this distribution, the distribution signaled that support is not available
+either for this operating system or this version of Perl.  Nevertheless, any
+diagnostic output produced is provided below for reference.  If this is not
 what you expect, you may wish to consult the CPAN Testers Wiki:
 
-http://cpantest.grango.org/wiki/CPANAuthorNotes
+http://wiki.cpantesters.org/wiki/CPANAuthorNotes
 HERE
-    
+
 );
 
 sub _report_text {
@@ -1019,16 +1068,16 @@ sub _report_text {
         $test_log .= "\n[Output truncated after $max_k]\n\n";
     }
     # Flag automated report
-    my $default_comment = $ENV{AUTOMATED_TESTING} 
-        ? "this report is from an automated smoke testing program\nand was not reviewed by a human for accuracy" 
+    my $default_comment = $ENV{AUTOMATED_TESTING}
+        ? "this report is from an automated smoke testing program\nand was not reviewed by a human for accuracy"
         : "none provided" ;
-        
+
     # generate report
     my $output = << "ENDREPORT";
 Dear $data->{author},
-    
+
 This is a computer-generated report for $data->{dist_name}
-on perl $data->{perl_version}, created by CPAN-Reporter-$CPAN::Reporter::VERSION\. 
+on perl $data->{perl_version}, created by CPAN-Reporter-$CPAN::Reporter::VERSION\.
 
 $intro_para{ $data->{grade} }
 Sections of this report:
@@ -1042,7 +1091,7 @@ Sections of this report:
 TESTER COMMENTS
 ------------------------------
 
-Additional comments from tester: 
+Additional comments from tester:
 
 $default_comment
 
@@ -1078,52 +1127,6 @@ ENDREPORT
     return $output;
 }
 
-#--------------------------------------------------------------------------#
-# _should_copy_author
-#--------------------------------------------------------------------------#
-
-sub _should_copy_author {
-    my ($result, $config) = @_;
-
-    # User prompts for action
-    my $author_email = $result->{author_id} 
-                     ? "$result->{author_id}\@cpan.org"
-                     : q{};
-    if ( ! $author_email ) {
-        $CPAN::Frontend->mywarn( "CPAN::Reporter: couldn't determine author_id and won't cc author.\n");
-        return;
-    }
-
-    # Skip if distribution name matches the cc_skipfile
-    if ( $config->{cc_skipfile} && -r $config->{cc_skipfile} ) {
-        my $cc_skipfile = IO::File->new( $config->{cc_skipfile}, "r" );
-        my $dist_id = $result->{dist}->pretty_id;
-        while ( my $pattern = <$cc_skipfile> ) {
-            chomp($pattern);
-            # ignore comments
-            next if substr($pattern,0,1) eq '#';
-            # if it doesn't match, continue with next pattern
-            next if $dist_id !~ /$pattern/i;
-            # if it matches, warn and return
-            $CPAN::Frontend->myprint( << "END_SKIP_DIST" );
-CPAN::Reporter: '$dist_id' matched against the cc_skipfile.  Won't copy author.
-END_SKIP_DIST
-            return;
-        }
-    }
-
-    # Don't copy author on perls with patchlevels
-    return if $Config{perl_patchlevel};
-
-    # Finally, prompt user if necessary
-    if ( _prompt( $config, "cc_author", $result->{grade}, "($author_email)?") =~ /^y/ ) {
-        return $author_email;
-    }
-    else {
-        return;
-    }
-}
-
 #--------------------------------------------------------------------------#
 # _special_vars_report
 #--------------------------------------------------------------------------#
@@ -1164,7 +1167,7 @@ sub _split_redirect {
 #--------------------------------------------------------------------------#
 # _temp_filename -- stand-in for File::Temp for backwards compatibility
 #
-# takes an optional prefix, adds 8 random chars and returns 
+# takes an optional prefix, adds 8 random chars and returns
 # an absolute pathname
 #
 # NOTE -- manual unlink required
@@ -1185,50 +1188,39 @@ sub _temp_filename {
 
 #--------------------------------------------------------------------------#
 # _timeout_wrapper
+# Timeout technique adapted from App::cpanminus (thank you Miyagawa!)
 #--------------------------------------------------------------------------#
 
 sub _timeout_wrapper {
     my ($cmd, $timeout) = @_;
-    
-    # Check Proc::ProcessTable also in case an unauthorized Proc::Killfam is
-    # present, as from Tk-ExecuteCommand
-    {
-      local $SIG{__WARN__} = sub {};  # protect against v-string warning
-      eval "require Proc::ProcessTable; require Proc::Killfam";
-    }
-    if ($@) {
-        $CPAN::Frontend->mywarn( << 'HERE' );
-CPAN::Reporter: you need Proc::ProcessTable and Proc::Killfam for 
-inactivity_timeout support.  Continuing without timeout...
-HERE
-        return;
-    }
 
     # protect shell quotes
     $cmd = quotemeta($cmd);
 
     my $wrapper = sprintf << 'HERE', $timeout, $cmd, $cmd;
 use strict;
-use Proc::Killfam 'killfam';
 my ($pid, $exitcode);
 eval {
-    local $SIG{CHLD};
-    local $SIG{ALRM} = sub {die 'Timeout'};
     $pid = fork;
-    die "Cannot fork: $!\n" unless defined $pid;
-    if ($pid) { #parent
+    if ($pid) {
+        local $SIG{CHLD};
+        local $SIG{ALRM} = sub {die 'Timeout'};
         alarm %s;
         my $wstat = waitpid $pid, 0;
+        alarm 0;
         $exitcode = $wstat == -1 ? -1 : $?;
-    } else {    #child
+    } elsif ( $pid == 0 ) {
+        setpgrp(0,0); # new process group
         exec "%s";
     }
+    else {
+      die "Cannot fork: $!\n" unless defined $pid;
+    }
 };
-alarm 0;
 if ($pid && $@ =~ /Timeout/){
-    killfam 9, $pid;
-    my $wstat = waitpid $pid, 0;
-    $exitcode = $wstat == -1 ? -1 : $?;
+    kill -9 => $pid; # and send to our child's whole process group
+    waitpid $pid, 0;
+    $exitcode = 9; # force result to look like SIGKILL
 }
 elsif ($@) {
     die $@;
@@ -1263,7 +1255,7 @@ HERE
                      split /$Config{path_sep}/, $ENV{PATH};
         if (! $path) {
             $CPAN::Frontend->mywarn( << "HERE" );
-CPAN::Reporter: can't locate $exe in the PATH. 
+CPAN::Reporter: can't locate $exe in the PATH.
 Continuing without timeout...
 HERE
             return;
@@ -1327,8 +1319,8 @@ sub _toolchain_report {
     $result->{toolchain} = $installed;
 
     my $mod_width = _max_length( keys %$installed );
-    my $ver_width = _max_length( 
-        map { $installed->{$_}{have} } keys %$installed  
+    my $ver_width = _max_length(
+        map { $installed->{$_}{have} } keys %$installed
     );
 
     my $format = "    \%-${mod_width}s \%-${ver_width}s\n";
@@ -1356,20 +1348,20 @@ sub _toolchain_report {
 # the way they would be found when test programs are run.  This means that
 # any updates to PERL5LIB will be reflected in the results.
 #
-# File-finding logic taken from CPAN::Module::inst_file().  Logic to 
+# File-finding logic taken from CPAN::Module::inst_file().  Logic to
 # handle newer Module::Build prereq syntax is taken from
 # CPAN::Distribution::unsat_prereq()
 #
 #--------------------------------------------------------------------------#
 
-my $version_finder = $INC{'CPAN/Reporter/PrereqCheck.pm'};
+my $version_finder = quotemeta($INC{'CPAN/Reporter/PrereqCheck.pm'});
 
 sub _version_finder {
     my %prereqs = @_;
 
     my $perl = Probe::Perl->find_perl_interpreter();
     my @prereq_results;
-    
+
     my $prereq_input = _temp_filename( 'CPAN-Reporter-PI-' );
     my $fh = IO::File->new( $prereq_input, "w" )
         or die "Could not create temporary '$prereq_input' for prereq analysis: $!";
@@ -1402,7 +1394,7 @@ sub _version_finder {
 __END__
 
 #--------------------------------------------------------------------------#
-# pod documentation 
+# pod documentation
 #--------------------------------------------------------------------------#
 
 =begin wikidoc
@@ -1454,7 +1446,7 @@ If CPAN.pm was upgraded, it needs to be reloaded.
 
 If upgrading from a very old version of CPAN.pm, users may be prompted to renew
 their configuration settings, including the 'test_report' option to enable
-CPAN::Reporter.  
+CPAN::Reporter.
 
 If not prompted automatically, users should manually initialize CPAN::Reporter
 support.  After enabling CPAN::Reporter, CPAN.pm will automatically continue
@@ -1485,7 +1477,7 @@ See [CPAN::Reporter::Config] for advanced configuration settings.
 == Using CPAN::Reporter
 
 Once CPAN::Reporter is enabled and configured, test or install modules with
-CPAN.pm as usual.  
+CPAN.pm as usual.
 
 For example, to force CPAN to repeat tests for CPAN::Reporter to see how it
 works:
@@ -1499,21 +1491,20 @@ addition information.
 
 CPAN::Reporter will assign one of the following grades to the report:
 
-* {pass} -- all tests were successful  
+* {pass} -- distribution built and tested correctly
+* {fail} --  distribution failed to test correctly
+* {unknown} -- distribution failed to build, had no test suite or outcome was
+inconclusive
+* {na} --- distribution is not applicable to this platform and/or
+version of Perl
 
-* {fail} -- one or more tests failed, one or more test files died during
-testing or no test output was seen
+In returning results of the test suite to CPAN.pm, "pass" and "unknown" are
+considered successful attempts to "make test" or "Build test" and will not
+prevent installation.  "fail" and "na" are considered to be failures and
+CPAN.pm will not install unless forced.
 
-* {na} -- tests could not be run on this platform or version of perl
-
-* {unknown} -- no test files could be found (either t/*.t or test.pl) or 
-a result could not be determined from test output (e.g tests may have hung 
-and been interrupted)
-
-In returning results to CPAN.pm, "pass" and "unknown" are considered successful
-attempts to "make test" or "Build test" and will not prevent installation.
-"fail" and "na" are considered to be failures and CPAN.pm will not install
-unless forced.
+An error from Makefile.PL/Build.PL or make/Build will also be graded as
+"unknown" and a failure will be signaled to CPAN.pm.
 
 If prerequisites specified in {Makefile.PL} or {Build.PL} are not available,
 no report will be generated and a failure will be signaled to CPAN.pm.
@@ -1524,20 +1515,20 @@ CPAN::Reporter includes information in the test report about environment
 variables and special Perl variables that could be affecting test results in
 order to help module authors interpret the results of the tests.  This includes
 information about paths, terminal, locale, user/group ID, installed toolchain
-modules (e.g. ExtUtils::MakeMaker) and so on.  
+modules (e.g. ExtUtils::MakeMaker) and so on.
 
 These have been intentionally limited to items that should not cause harmful
 personal information to be revealed -- it does ~not~ include your entire
 environment.  Nevertheless, please do not use CPAN::Reporter if you are
-concerned about the disclosure of this information as part of your test report.  
+concerned about the disclosure of this information as part of your test report.
 
-Users wishing to review this information may choose to edit the report 
+Users wishing to review this information may choose to edit the report
 prior to sending it.
 
 = BUGS
 
-Please report any bugs or feature using the CPAN Request Tracker.  
-Bugs can be submitted through the web interface at 
+Please report any bugs or feature using the CPAN Request Tracker.
+Bugs can be submitted through the web interface at
 [http://rt.cpan.org/Dist/Display.html?Queue=CPAN-Reporter]
 
 When submitting a bug or request, please include a test-file or a patch to an
@@ -1548,10 +1539,10 @@ existing test-file that illustrates the bug or desired feature.
 Information about CPAN::Testers:
 
 * [CPAN::Testers] -- overview of CPAN Testers architecture stack
-* [http://cpantesters.perl.org] -- project home with all reports
-* [http://cpantest.grango.org] -- documentation and wiki
+* [http://www.cpantesters.org] -- project home with all reports
+* [http://wiki.cpantesters.org] -- documentation and wiki
 
-Additional Documentation: 
+Additional Documentation:
 
 * [CPAN::Reporter::Config] -- advanced configuration settings
 * [CPAN::Reporter::FAQ] -- hints and tips
@@ -1566,7 +1557,7 @@ Copyright (c) 2006, 2007, 2008 by David A. Golden
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
-You may obtain a copy of the License at 
+You may obtain a copy of the License at
 [http://www.apache.org/licenses/LICENSE-2.0]
 
 Unless required by applicable law or agreed to in writing, software
