@@ -1,8 +1,8 @@
 package CPAN::Reporter::Config;
 use strict; 
 use vars qw/$VERSION/;
-$VERSION = '1.16_51'; 
-$VERSION = eval $VERSION;
+$VERSION = '1.18_05';
+$VERSION = eval $VERSION; ## no critic
 
 use Config::Tiny ();
 use File::HomeDir (); 
@@ -239,16 +239,6 @@ the test report should be sent before actually emailing the
 report.  This gives the opportunity to bypass sending particular
 reports if you need to (e.g. if you caused the failure).
 This option takes "grade:action" pairs.
-HERE
-    },
-    cc_author => {
-        default => 'default:yes pass/na:no',
-        prompt => "Do you want to CC the module author ", # (author@cpan.org) added dynamically
-        validate => \&_validate_grade_action_pair,
-        info => <<'HERE',
-If you would like, CPAN::Reporter will copy the module author with
-the results of your tests.  By default, authors are copied only on 
-failed/unknown results. This option takes "grade:action" pairs.  
 HERE
     },
     send_duplicates => {
@@ -318,11 +308,15 @@ sub _get_config_dir {
     ) {
         return $ENV{PERL_CPAN_REPORTER_DIR};
     }
-    else {
-        return ( $^O eq 'MSWin32' )
-            ? File::Spec->catdir(File::HomeDir->my_documents, ".cpanreporter")
-            : File::Spec->catdir(File::HomeDir->my_home, ".cpanreporter") ;
+
+    my $conf_dir = File::Spec->catdir(File::HomeDir->my_home, ".cpanreporter");
+
+    if ($^O eq 'MSWin32') {
+      my $alt_dir = File::Spec->catdir(File::HomeDir->my_documents, ".cpanreporter");
+      $conf_dir = $alt_dir if -d $alt_dir && ! -d $conf_dir;
     }
+
+    return $conf_dir;
 }
 
 #--------------------------------------------------------------------------#
@@ -542,14 +536,13 @@ From the CPAN shell:
 = DESCRIPTION
 
 Default options for CPAN::Reporter are read from a configuration file 
-{.cpanreporter/config.ini} in the user's home directory (Unix and OS X)
-or "My Documents" directory (Windows).
+{.cpanreporter/config.ini} in the user's home directory.
 
 The configuration file is in "ini" format, with the option name and value
 separated by an "=" sign
 
   email_from = "John Doe" <johndoe@nowhere.org>
-  cc_author = no
+  edit_report = no
 
 Interactive configuration of email address, mail server and common
 action prompts may be repeated at any time from the CPAN shell.  
@@ -658,22 +651,13 @@ These additional options are only necessary in special cases, for example if
 the default editor cannot be found or if reports shouldn't be sent in 
 certain situations or for automated testing, and so on.
 
-* {cc_author = <grade:action> ...} -- should module authors should be sent a 
-copy of the test report at their {author@cpan.org} address? 
-(default:yes pass/na:no)
-* {cc_skipfile = <skipfile>} -- filename containing regular expressions (one
-per line) to match against the distribution ID (e.g. 
-'AUTHOR/Dist-Name-0.01.tar.gz'); the author will not be copied if a match is 
-found regardless of cc_author; non-absolute filename must be in the .cpanreporter 
-config directory;
 * {command_timeout} -- if greater than zero and the CPAN config is
 {inactivity_timeout} is not set, then any commands executed by CPAN::Reporter 
 will be halted after this many seconds; useful for unattended smoke testing 
 to stop after some amount of time; generally, this should be large -- 
 900 seconds or more -- as some distributions' tests take quite a long time to 
 run.  On MSWin32, [Win32::Job] is a needed and trying to kill a processes may 
-actually deadlock in some situations -- so use at your own risk.  On other
-platforms, [Proc::ProcessTable] and [Proc::Killfam] are required.
+actually deadlock in some situations -- so use at your own risk.
 * {editor = <editor>} -- editor to use to edit the test report; if not set,
 Test::Reporter will use environment variables {VISUAL}, {EDITOR} or {EDIT}
 (in that order) to find an editor 
@@ -685,8 +669,10 @@ reports be sent, regardless of {send_report}? (default:no)
 {send_report} during the make phase
 * {send_test_report = <grade:action> ...} -- if defined, used in place of 
 {send_report} during the test phase
-* {send_skipfile = <skipfile>} -- like {cc_skipfile} but no report will be 
-sent at all if a match is found
+* {send_skipfile = <skipfile>} -- filename containing regular expressions (one
+per line) to match against the distribution ID (e.g. 
+'AUTHOR/Dist-Name-0.01.tar.gz'); the report will not be sent if a match is 
+found; non-absolute filename must be in the .cpanreporter config directory;
 * {transport = <transport> [transport args]} -- if defined, passed to the 
 {transport()} method of [Test::Reporter].  See below for 
 more details.  (CPAN::Reporter uses 'Net::SMTP' for this by default.)
@@ -726,6 +712,7 @@ set the 'transport' config option to the name of the transport module
 (without the 'Test::Reporter::Transport' prefix) and any required arguments,
 separated by white space. For example:
 
+  transport=Net::SMTP Port 587
   transport=Net::SMTP::TLS User jdoe@example.com Password 12345
   transport=HTTPGateway http://example.com/cpantesters.cgi MyKey
   transport=File ~/saved-reports-dir
